using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using BibliotecaComum;

namespace AmbienteDeDesenvolvimento
{
    public class Projeto
    {
        private string _enderecoDir;
        private string _enderecoCfg;
        private ArquivoIni _ini;
        private bool _existia;

        public bool Existia
        {
            get { return _existia; }
        }


        public Projeto(string diretorio)
        {
            _enderecoDir = diretorio;
            if (!Directory.Exists(diretorio))
            {
                Directory.CreateDirectory(diretorio);
            }
            _enderecoCfg = Path.Combine(diretorio, "projeto.hpj");
            _existia = File.Exists(_enderecoCfg);
            _ini = new ArquivoIni(_enderecoCfg);

        }

        public string EnderecoDiretorio
        {
            get
            {
                return _enderecoDir;
            }
        }

        public string ArquivoConfiguracao
        {
            get
            {
                return Path.GetFileName(_enderecoCfg);
            }
        }

        public string Caminho(string arquivo)
        {
            return Path.Combine(_enderecoDir, arquivo);
        }

        /// <summary>
        /// Retorna a extenso sem pontos e caixa baixa.
        /// </summary>
        /// <param name="arquivo">Nome do arquivo</param>
        /// <returns>Extenso sem pontos e caixa baixa</returns>
        public string Extensao(string arquivo)
        {
            return Path.GetExtension(arquivo).ToLower().Replace(".", "");
        }

        public string Nome
        {
            get
            {
                return _ini.Ler("Projeto", "Nome", "Sem nome");
            }
            set
            {
                _ini.Gravar("Projeto", "Nome", value);
            }
        }

        public string Montador
        {
            get
            {
                return _ini.Ler("Projeto", "Montador", "osasm");
            }
            set
            {
                _ini.Gravar("Projeto", "Montador", value);
            }
        }

        public TipoDeProjeto Tipo
        {
            get
            {
                string tipo = _ini.Ler("Projeto", "Tipo", "husis");
                switch (tipo)
                {
                    case "husis":
                        return TipoDeProjeto.ProgramaHUSIS;
                    case "bin":
                        return TipoDeProjeto.BinarioSimples;
                    case "com":
                        return TipoDeProjeto.BinarioDOS;
                    default:
                        throw new NotImplementedException("Tipo de Projeto no suportado");
                }
            }
            set
            {
                string tipo = "";
                switch (value)
                {
                    case TipoDeProjeto.BinarioSimples:
                        tipo = "bin";
                        break;
                    case TipoDeProjeto.ProgramaHUSIS:
                        tipo = "husis";
                        break;
                    case TipoDeProjeto.BinarioDOS:
                        tipo = "com";
                        break;
                    default:
                        break;
                }
                _ini.Gravar("Projeto", "Tipo", tipo);
            }
        }

        public bool TipoHusis
        {
            get
            {
                return Tipo == TipoDeProjeto.ProgramaHUSIS;
            }
            set
            {
                if (value) Tipo = TipoDeProjeto.ProgramaHUSIS;
            }
        }

        public bool TipoBinario
        {
            get
            {
                return Tipo == TipoDeProjeto.BinarioSimples;
            }
            set
            {
                if (value) Tipo = TipoDeProjeto.BinarioSimples;
            }
        }

        public bool TipoBinarioDOS
        {
            get
            {
                return Tipo == TipoDeProjeto.BinarioDOS;
            }
            set
            {
                if (value) Tipo = TipoDeProjeto.BinarioDOS;
            }
        }

        public string RotinaInicial
        {
            get
            {
                return _ini.Ler("Projeto", "RotinaInicial", null);
            }
            set
            {
                _ini.Gravar("Projeto", "RotinaInicial", value);
            }
        }


        public string ArquivoPrincipal
        {
            get
            {
                return _ini.Ler("Projeto", "Arquivo", null);
            }
            set
            {
                _ini.Gravar("Projeto", "Arquivo", value);
            }
        }

        public string ExtensaoAssembly
        {
            get
            {
                return _ini.Ler("Projeto", "ExtensaoAssembly", "s");
            }
            set
            {
                _ini.Gravar("Projeto", "ExtensaoAssembly", value);
            }
        }

        public string ComandoExterno
        {
            get
            {
                return _ini.Ler("Projeto", "Externo", null);
            }
            set
            {
                _ini.Gravar("Projeto", "Externo", value);
            }
        }

        public string[] Arquivos
        {
            get
            {
                return _ini.Listar("Arquivos");
            }
        }

        public AcaoArquivo ParaAcaoArquivo(string acao)
        {
            switch (acao.ToLower().Trim())
            {
                case "":
                    return AcaoArquivo.Nenhuma;
                case "asm":
                    return AcaoArquivo.CompilarAssembly;
                case "trad":
                    return AcaoArquivo.CompilarTraducao;
                default:
                    throw new NotImplementedException("Ao de arquivo no implementada");
            }
        }

        public string DeAcaoArquivo(AcaoArquivo acao)
        {
            switch (acao)
            {
                case AcaoArquivo.Nenhuma:
                    return "";
                case AcaoArquivo.CompilarAssembly:
                    return "asm";
                case AcaoArquivo.CompilarTraducao:
                    return "trad";
                default:
                    throw new NotImplementedException("Ao de arquivo no implementada");
            }
        }


        public void Adicionar(string arquivo, AcaoArquivo acao)
        {
            _ini.Gravar("Arquivos", arquivo, DeAcaoArquivo(acao));
        }

        public AcaoArquivo LerAcaoArquivo(string arquivo)
        {
            return ParaAcaoArquivo(_ini.Ler("Arquivos", arquivo, ""));
        }

        public void GravarAcaoArquivo(string arquivo, AcaoArquivo acao)
        {
            _ini.Gravar("Arquivos", arquivo, DeAcaoArquivo(acao));
        }

        public void Remover(string arquivo)
        {
            _ini.Remover("Arquivos", arquivo);
        }


        internal void Salvar()
        {
            _ini.SalvarArquivo();
        }
    }

    public enum TipoDeProjeto
    {
        BinarioSimples,
        BinarioDOS,
        ProgramaHUSIS
    }

    public enum AcaoArquivo
    {
        Nenhuma,
        CompilarAssembly,
        CompilarTraducao
    }
}
